/*
fs.c -- filesystem access routines
Copyright (C) 2005  Martin Wolters

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the 

Free Software Foundation, Inc.
51 Franklin Street, Fifth Floor
Boston, MA  02110-1301
USA

You can contact the author via electronic mail by the address
'nc-wolterma4@netcologne.de'.
*/

#include <stdio.h>
#include <dirent.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#include "fs.h"
#include "video.h"

int iReadConfig(char *cConfFile, RULES *sRuleInfo) {
	FILE *fConfFile;
	int i=1, iRule=0;
	
//	printf("Opening config file '%s'...\n", cConfFile);
	fConfFile = fopen(cConfFile, "r");
	if(fConfFile == NULL) {
//		printf("Could not open config File.\n");
		return -1;
	}

	while((iRule < MAXRULES) &&
		  (i > 0)) {
		sRuleInfo->cExt[iRule] = (char*)malloc(512);
		sRuleInfo->cApp[iRule] = (char*)malloc(512);

		i = fscanf(fConfFile, "%s %s", 
			sRuleInfo->cExt[iRule], 
			sRuleInfo->cApp[iRule]);
		iRule++;
	}
//	printf("Read %d rules.\n", iRule);
	sRuleInfo->iNumRules = iRule -1;
	return 1;
}

int iGetRule(char *cFileName, RULES *sRuleInfo, char *cRule) {
	int iRule, iLen;
	char *cExt=(char*)malloc(NAMLEN);
	char *cApp=(char*)malloc(NAMLEN);

	iLen=strlen(cFileName);

	for(iRule = 0; iRule < sRuleInfo->iNumRules; iRule++) {
		cExt = cFileName + iLen - (strlen(sRuleInfo->cExt[iRule]));
		if(!strcmp(cExt, sRuleInfo->cExt[iRule])) {
			if(strcmp(sRuleInfo->cApp[iRule], "*") ==0)
				sprintf(cRule, "./%s", cFileName);
			else
				sprintf(cRule, "%s", sRuleInfo->cApp[iRule]);
			return iRule;
		}
	}
	return -1;
}

int iExecute(INFO *sInfo, DIRINFO *sDirInfo, RULES *sRuleInfo) {
	int i = sInfo->iCurLine + sInfo->iWndLine - STARTLIN;
	int iRule;
	char *cRule=(char*)malloc(512);

	if(sDirInfo->iFileType[i] == TYPE_DIR) {
		chdir(sDirInfo->cFileName[i]);
		iGetWd(sDirInfo);
		sInfo->iCurLine = STARTLIN;
		sInfo->iWndLine = 0;
		iUpdateScrn(sInfo, sDirInfo);
	} else if((iRule = iGetRule(sDirInfo->cFileName[i], sRuleInfo, cRule))>-1) {
		printf("Exec: %s %s\n", cRule, sDirInfo->cFileName[i]);
		execl(cRule, cRule,
			sDirInfo->cFileName[i], NULL);
	}
	
	return 0;
}

int iSortDir(DIRINFO *sDirInfo) {
	DIRINFO sBuf;
	int i=0,j;

	iInitDirInfo(&sBuf);
	strcpy(sBuf.cWorkingDir, sDirInfo->cWorkingDir);

	for(j=0; j < sDirInfo->iNumFiles; j++) {
		if(sDirInfo->iFileType[j] == TYPE_DIR) {
			strcpy(sBuf.cFileName[i], sDirInfo->cFileName[j]);
			sBuf.iFileType[i] = TYPE_DIR;
			i++;
		}							
	}
	
	for(j=0; j < sDirInfo->iNumFiles; j++) {
		if(sDirInfo->iFileType[j] == TYPE_FIL) {
			strcpy(sBuf.cFileName[i], sDirInfo->cFileName[j]);
			sBuf.iFileType[i] = TYPE_FIL;
			i++;
		}							
	}

	sBuf.iNumFiles=i;
	memcpy(sDirInfo, &sBuf, sizeof(DIRINFO));

	return i;
}

int iGetWd(DIRINFO *sDirInfo) {
	DIR *dDir;
	struct dirent *dEntry;
	int iNumFile=0;
	
	dDir = opendir(".");
	if((dDir == NULL) ||
	   (getcwd(sDirInfo->cWorkingDir, NAMLEN) == NULL)) {
		printf("Error reading working directory.\n");
		return -1;
	} else {
		while((dEntry = readdir(dDir)) &&
			  (iNumFile < MAXFILES)) {
			strncpy(sDirInfo->cFileName[iNumFile], dEntry->d_name, NAMLEN);
			if(dEntry->d_type == DT_DIR)
				sDirInfo->iFileType[iNumFile]=TYPE_DIR;
			else
				sDirInfo->iFileType[iNumFile]=TYPE_FIL;
			iNumFile++;
		}
		closedir(dDir);
		sDirInfo->iNumFiles = iNumFile;
		iSortDir(sDirInfo);
	}

	return sDirInfo->iNumFiles;
}

int iInitDirInfo(DIRINFO *sDirInfo) {
    int i;

	sDirInfo->cWorkingDir=(char*)malloc(NAMLEN);    
    for(i=0;i<MAXFILES;i++)
        sDirInfo->cFileName[i]=(char*)malloc(NAMLEN);
        
    return i;
}  

/*)\
\(*/
