/*
unifier -- unified frontend for data access
Copyright (C) 2005  Martin Wolters

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the 

Free Software Foundation, Inc.
51 Franklin Street, Fifth Floor
Boston, MA  02110-1301
USA

You can contact the author via electronic mail by the address
'nc-wolterma4@netcologne.de'.
*/

#include <SDL.h>
#include <SDL_ttf.h>
#include <stdio.h>
#include <malloc.h>

#include "defs.h"
#include "fs.h"
#include "video.h"

int iObeyKeys(INFO *sInfo, DIRINFO *sDirInfo, RULES *sRuleInfo) {
	SDL_Color sColor;
	int iMaxLine = 240 / SIZE - 1;
	
		if(sInfo->sKeyInfo.iDown) {
		   if((sInfo->iCurLine + sInfo->iWndLine - STARTLIN) < 
			  (sDirInfo->iNumFiles - 1)) {
				if((sInfo->iCurLine < iMaxLine)) {
					sColor = sInfo->sBgCol;
					sInfo->iLine = sInfo->iCurLine;
					iWriteText("->", sInfo, sColor);
			
					sInfo->iCurLine++;
				} else {
					sInfo->iWndLine++;
					iUpdateScrn(sInfo, sDirInfo);
		   		}
			}
		}

		if(sInfo->sKeyInfo.iPgDown) {
			if((sInfo->iWndLine + iMaxLine) < (sDirInfo->iNumFiles - 1)) {
				sInfo->iCurLine = STARTLIN;
				iUpdateScrn(sInfo, sDirInfo);
			}
		}
		
		if(sInfo->sKeyInfo.iUp) {
			if(sInfo->iCurLine > STARTLIN) {
				sColor = sInfo->sBgCol;
				sInfo->iLine = sInfo->iCurLine;
				iWriteText("->", sInfo, sColor);
	
				sInfo->iCurLine--;
			} else if(sInfo->iWndLine > 0) {
				sInfo->iWndLine--;
				iUpdateScrn(sInfo, sDirInfo);
			}
		}
		
		if(sInfo->sKeyInfo.iPgUp) {
			sInfo->iWndLine -= iMaxLine;
			if(sInfo->iWndLine < 0)
				sInfo->iWndLine = 0;
			sInfo->iCurLine = STARTLIN;
				iUpdateScrn(sInfo, sDirInfo);
		}
		
		// Change to the parent directory
		if(sInfo->sKeyInfo.iParent) {
			sInfo->iWndLine = 0;
			sInfo->iCurLine = 1 + STARTLIN;
			sInfo->sKeyInfo.iExecute = 1;
		}
	
		// Handle the selected file/diretory
		if(sInfo->sKeyInfo.iExecute) {
			sInfo->sKeyInfo.iExecute = 0;
			iExecute(sInfo, sDirInfo, sRuleInfo);
		}
		
		// Quit :(
		if(sInfo->sKeyInfo.iQuit)
			// hedoesntlikemesoilldeleteallhisfiles();
			return 1;

	sColor = sInfo->sFnCol[0];
	sInfo->iLine = sInfo->iCurLine;
	iWriteText("->", sInfo, sColor);

	return 0;
}

int iHandleEvent(INFO *sInfo, DIRINFO *sDirInfo, RULES *sRuleInfo, int iType) {
	int i=0;
	sInfo->iColumn = 0;

#ifdef GP2X
	if(iType == SDL_JOYBUTTONDOWN)
#else
	if(iType == SDL_KEYDOWN)
#endif	
		i = 1;

	switch(sInfo->iEvent) {
		case BTN_UP:	sInfo->sKeyInfo.iUp=i;		break;
		case BTN_DN:	sInfo->sKeyInfo.iDown=i;	break;	
		case BTN_PGUP:	sInfo->sKeyInfo.iPgUp=i;	break;
		case BTN_PGDN:	sInfo->sKeyInfo.iPgDown=i;	break;
		case BTN_DIR:	sInfo->sKeyInfo.iParent=i;	break;
		case BTN_OK:
		case BTN_OK2:	sInfo->sKeyInfo.iExecute=i;	break;
		case BTN_QU:	sInfo->sKeyInfo.iQuit=i;	break;
	}
	
	return 0;

}

int iEvalEvents(INFO *sInfo, DIRINFO *sDirInfo, RULES *sRuleInfo) {
	SDL_Event sEvent;
	
	int iRet = 0;
	while(SDL_PollEvent(&sEvent)) {
		switch(sEvent.type) {
#ifdef GP2X
			case SDL_JOYBUTTONUP:
			case SDL_JOYBUTTONDOWN:
				sInfo->iEvent = sEvent.jbutton.button;
#else
			case SDL_KEYUP:
			case SDL_KEYDOWN:
				sInfo->iEvent = sEvent.key.keysym.sym;
#endif
				iHandleEvent(sInfo, sDirInfo, sRuleInfo, sEvent.type);
				break;
		}
		iRet=iObeyKeys(sInfo, sDirInfo, sRuleInfo);
	}
	return iRet;
}

int iInitInfo(INFO *sInfo) {
	SDL_Color sFnCol = { 88, 118, 163};
	SDL_Color sDrCol = {240, 240, 240};
	SDL_Color sBgCol = {128, 172, 239};
	
	sInfo->sFnCol[TYPE_FIL] = sFnCol;
	sInfo->sFnCol[TYPE_DIR] = sDrCol;
	sInfo->sBgCol = sBgCol;
	sInfo->iLine = 0;
	sInfo->iCurLine = STARTLIN;
	sInfo->iWndLine = 0;

	sInfo->sKeyInfo.iUp=0;
	sInfo->sKeyInfo.iDown=0;
	sInfo->sKeyInfo.iPgUp=0;
	sInfo->sKeyInfo.iPgDown=0;
	sInfo->sKeyInfo.iParent=0;
	sInfo->sKeyInfo.iExecute=0;
	sInfo->sKeyInfo.iQuit=0;
}

int main() {
	DIRINFO sDirInfo;
	INFO sInfo;
	RULES sRuleInfo;
	
	int iStop = 0;
	
	// set up our window
	if((sInfo.dWindow = dVidInit()) == NULL) {
		iVidStop();
		return -1;
	}

	if((sInfo.fFont = fFontInit("Vera.ttf")) == NULL) {
		iVidStop();
		return -1;
	}
	
	// initialize global variables
	iInitInfo(&sInfo);

	iReadConfig("unify.conf", &sRuleInfo);
	iInitDirInfo(&sDirInfo);
	iGetWd(&sDirInfo);
	
	iUpdateScrn(&sInfo, &sDirInfo);
	while(!iStop) {
		iStop = iEvalEvents(&sInfo, &sDirInfo, &sRuleInfo);
	}
	
	iVidStop();
	
	return 0;
}

/*)\
\(*/
